@@ -21,8 +21,9 @@ import random
 import sys
 import traceback
 
-import gtk
-import gobject
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import GObject
 
 from solfege import gu
 from solfege import inputwidgets
@@ -55,17 +56,23 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         self.m_lessonfile = None
         self.m_question = None
         self.m_custom_mode = False
-        self.m_lessonfile_defs = {'yes': True, 'no': False}
+        self.m_lessonfile_defs = {'yes': True, 'no': False,
+            'hidden': 'hidden',
+            'enabled': 'enabled',
+            'disabled': 'disabled'}
+        # Some exercises modules want other default values for the
+        # lessonfile header variables. They can add them to this dict
+        self.m_lessonfile_header_defaults = {}
     def maybe_auto_new_question(self):
         if self.get_bool('new_question_automatically'):
             if self.m_timeout_handle is None:
                 def remove_timeout(self=self):
                     self.m_timeout_handle = None
                     self.g_view.new_question()
-                self.m_timeout_handle = gobject.timeout_add(int(self.get_float('seconds_before_new_question')*1000),  remove_timeout)
+                self.m_timeout_handle = GObject.timeout_add(int(self.get_float('seconds_before_new_question')*1000),  remove_timeout)
     def end_practise(self):
         if self.m_timeout_handle:
-            gobject.source_remove(self.m_timeout_handle)
+            GObject.source_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
         self.q_status = self.QSTATUS_NO
         soundcard.synth.stop()
@@ -89,7 +96,8 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         if not self.m_lessonfile:
             self.m_P = None
             return
-        self.m_P = self.lessonfileclass(self.m_lessonfile_defs)
+        self.m_P = self.lessonfileclass(self.m_lessonfile_defs,
+                                        self.m_lessonfile_header_defaults)
         self.m_P.parse_file(self.m_lessonfile)
         if [q for q in self.m_P.m_questions if isinstance(q.get('music'), lessonfile.Cmdline)]:
             run = gu.dialog_yesno(_("The lessonfile contain potentially dangerous code because it run external programs. Run anyway?"))
@@ -112,9 +120,13 @@ class MelodicIntervalTeacher(Teacher):
     When this class was created it was used by melodicinterval
     and singinterval.
     """
+    class ConfigureException(Exception):
+        def __init__(self, msg=_("The exercise has to be better configured.")):
+            Exception.__init__(self, msg)
     OK = 0
     ERR_PICKY = 1
     ERR_CONFIGURE = 2
+    no_intervals_str = _("No intervals selected for interval number %i.")
     def __init__(self, exname):
         Teacher.__init__(self, exname)
         self.m_tonika = None
@@ -122,13 +134,9 @@ class MelodicIntervalTeacher(Teacher):
     def new_question(self, L, H):
         assert isinstance(L, basestring)
         assert isinstance(H, basestring)
-        if self.get_list('ask_for_intervals_0') == []:
-            return self.ERR_CONFIGURE
-        L, H = utils.adjust_low_high_to_irange(L, H,
-                     self.get_list('ask_for_intervals_0'))
 
         if self.m_timeout_handle:
-            gobject.source_remove(self.m_timeout_handle)
+            GObject.source_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
 
         if solfege.app.m_test_mode:
@@ -137,7 +145,6 @@ class MelodicIntervalTeacher(Teacher):
                 old_toptone = old_tonika + self.m_question[0]
             self.m_P.next_test_question()
             self.m_question = [self.m_P.m_test_questions[self.m_P.m_test_idx]]
-            #FIXME use tone pitch range from preferences window.
             self.m_tonika = mpd.MusicalPitch()
             # Do this loop to make sure two questions in a row does not have
             # the same top or bottom tone.
@@ -159,64 +166,61 @@ class MelodicIntervalTeacher(Teacher):
         last_tonika = self.m_tonika
         last_question = self.m_question
         if self.get_bool("lock-to-key"):
+            # We don't have to check the validity of these two variables
+            # because get_int will check that a int value is stored in the
+            # database (and make it an int if it is not), and nComboBox
+            # will make sure the int value is within the correct limits.
             lock_tonic = mpd.MusicalPitch.new_from_int(self.get_int("lock-to-key-note"))
-            lock_scaletype = solfege.utils.random_tonic_and_interval_in_key.key_data.keys()[:][self.get_int("lock-to-key-scaletype")]
+            lock_scaletype = utils.key_data.keys()[:][self.get_int("lock-to-key-scaletype")]
 
-        for x in range(10):# we try max 10 times to get a question that
-                           # is different from the last one.
-            if self.get_bool("lock-to-key"):
-                self.m_tonika, i = \
-                    utils.random_tonic_and_interval_in_key(L, H,
-                        self.get_list('ask_for_intervals_0'),
-                        lock_tonic, lock_scaletype)
-            else:
-                self.m_tonika, i = utils.random_tonika_and_interval(L, H,
+        def try_make_question():
+            try:
+                if self.get_bool("lock-to-key"):
+                    self.m_tonika, i = \
+                        utils.random_tonic_and_interval_in_key(L, H,
+                            self.get_list('ask_for_intervals_0'),
+                            lock_tonic, lock_scaletype)
+                else:
+                    self.m_tonika, i = utils.random_tonika_and_interval(L, H,
                             self.get_list('ask_for_intervals_0'))
+            except utils.NoPossibleIntervals, e:
+                raise self.ConfigureException(self.no_intervals_str % 1)
+
             self.m_question = [i]
             t = self.m_tonika + i
             for x in range(1, self.get_int('number_of_intervals=1')):
-                if not self.get_list('ask_for_intervals_%i' % x):
-                    return self.ERR_CONFIGURE
+                interval_list = self.get_list('ask_for_intervals_%i' % x)
+                if not interval_list:
+                    raise self.ConfigureException(self.no_intervals_str % (x + 1))
                 if self.get_bool('lock-to-key'):
                     i = utils.random_interval_in_key(t, L, H,
-                            self.get_list('ask_for_intervals_%i' % x),
+                            interval_list,
                             lock_tonic, lock_scaletype)
                     if not i:
-                        return self.ERR_CONFIGURE
+                        raise self.ConfigureException(_(u"Failed to select random interval number %i because of the configuration of the exercise. Either you have enabled intervals only if one direction, or none of the intervals belong to the key selected when you enabled «Lock to key».") % x)
                 else:
                     i = utils.random_interval(t, L, H,
                             self.get_list('ask_for_intervals_%i' % x))
-                if not i:
-                    # if we can't find an interval that is with the range
-                    # we, find the interval that is closest to the range
-                    # of notes the user want. This mean that the questions
-                    # are not necessarily that random.
-                    low = mpd.MusicalPitch.new_from_int(L)
-                    high = mpd.MusicalPitch.new_from_int(H)
-                    off = 1000
-                    best = None
-                    for interval in self.get_list('ask_for_intervals_%i'%x):
-                        try:
-                            if t + interval > high:
-                                if t + interval - high < off:
-                                    off = t + interval - high
-                                    best = interval
-                            if t + interval < low:
-                                if low - (t + interval) < off:
-                                    off = low - (t + interval)
-                                    best = interval
-                        except ValueError:
-                            return self.ERR_CONFIGURE
-                    i = best
+                    if not i:
+                        raise self.ConfigureException(_("Failed to select random interval number %i because of the configuration of the exercise. You should select some intervals going in both directions.") % x)
                 self.m_question.append(i)
                 t = t + i
+        max_try = 1000
+        for counter in range(max_try):
+            try:
+                try_make_question()
+            except self.ConfigureException, e:
+                if counter == max_try -1:
+                    raise
+                else:
+                    continue
             if last_tonika is not None \
                     and last_tonika == self.m_tonika \
                     and last_question == self.m_question:
                 continue
-            break
-        self.q_status = self.QSTATUS_NEW
-        return self.OK
+            self.q_status = self.QSTATUS_NEW
+            return self.OK
+
     def play_question(self):
         if self.q_status == self.QSTATUS_NO:
             return
@@ -240,7 +244,7 @@ class RhythmAddOnClass:
                self.ERR_NO_ELEMS : if no elements are set to be practised.
         """
         if self.m_timeout_handle:
-            gobject.source_remove(self.m_timeout_handle)
+            GObject.source_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
 
         if self.get_bool('config/picky_on_new_question') \
@@ -324,7 +328,7 @@ class RhythmAddOnClass:
             else:
                 self.set_int(n, default)
 
-class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
+class Gui(Gtk.VBox, cfg.ConfigUtils, QstatusDefs):
     """Important members:
          - practise_box
          - action_area
@@ -332,71 +336,71 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
     """
     short_delay = 700
     def __init__(self, teacher, no_notebook=False):
-        gtk.VBox.__init__(self)
+        Gtk.VBox.__init__(self)
         cfg.ConfigUtils.__init__(self, teacher.m_exname)
         assert type(no_notebook) == bool
         self._std_buttons = []
         self.m_key_bindings = {}
         self.m_t = teacher
 
-        vbox = gtk.VBox()
+        vbox = Gtk.VBox()
         vbox.set_spacing(gu.PAD)
         vbox.set_border_width(gu.PAD)
         vbox.show()
 
-        self.practise_box = gtk.VBox()
+        self.practise_box = Gtk.VBox()
         self.practise_box.show()
-        vbox.pack_start(self.practise_box, False)
+        vbox.pack_start(self.practise_box, True, True, 0)
 
-        box = gtk.VBox()
-        self.practise_box.pack_start(box, False, padding=gu.hig.SPACE_LARGE)
+        box = Gtk.VBox()
+        self.practise_box.pack_start(box, False, False, padding=gu.hig.SPACE_LARGE)
         box.show()
-        self.g_lesson_heading = gtk.Label()
-        box.pack_start(self.g_lesson_heading)
+        self.g_lesson_heading = Gtk.Label()
+        box.pack_start(self.g_lesson_heading, True, True, 0)
 
-        self.g_lesson_description = gtk.Label()
+        self.g_lesson_description = Gtk.Label()
         self.g_lesson_description.set_line_wrap(True)
-        box.pack_start(self.g_lesson_description)
+        box.pack_start(self.g_lesson_description, True, True, 0)
 
-        self.action_area = gtk.HBox()
+        self.action_area = Gtk.HBox()
         self.action_area.show()
-        vbox.pack_start(self.action_area, False)
+        vbox.pack_start(self.action_area, False, False, 0)
 
-        self.config_box = gtk.VBox()
+        self.config_box = Gtk.VBox()
         self.config_box.set_border_width(gu.PAD)
         self.config_box.show()
-        self.config_box_sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        self.config_box_sizegroup = Gtk.SizeGroup(Gtk.SizeGroupMode.HORIZONTAL)
         if no_notebook:
-            self.pack_start(vbox)
-            self.pack_start(self.config_box, False)
+            self.pack_start(vbox, True, True, 0)
+            self.pack_start(self.config_box, False, False, 0)
             self.g_notebook = None
         else:
-            self.g_notebook = gtk.Notebook()
-            self.pack_start(self.g_notebook)
+            self.g_notebook = Gtk.Notebook()
+            self.pack_start(self.g_notebook, True, True, 0)
 
-            self.g_notebook.append_page(vbox, gtk.Label(_("Practise")))
-            self.g_notebook.append_page(self.config_box, gtk.Label(_("Config")))
+            self.g_notebook.append_page(vbox, Gtk.Label(label=_("Practise")))
+            self.g_notebook.append_page(self.config_box, Gtk.Label(label=_("Config")))
             self.g_notebook.show()
-        self.g_cancel_test = gtk.Button(_("_Cancel test"))
+        self.g_cancel_test = Gtk.Button(_("_Cancel test"))
         self.g_cancel_test.connect('clicked', self.on_cancel_test)
-        self.action_area.pack_end(self.g_cancel_test, False)
+        self.action_area.pack_end(self.g_cancel_test, False, False, 0)
     def add_module_is_deprecated_label(self):
         """
         The deprecated module must set a message in self.g_deprecated_label
         in on_start_practise, preferable telling the file name of the
         lesson file.
         """
-        img = gtk.Image()
-        img.set_from_stock(gtk.STOCK_DIALOG_WARNING,
-                           gtk.ICON_SIZE_BUTTON)
-        hbox = gtk.HBox()
+        img = Gtk.Image()
+        img.set_from_stock(Gtk.STOCK_DIALOG_WARNING,
+                           Gtk.IconSize.BUTTON)
+        hbox = Gtk.HBox()
         hbox.set_border_width(12)
         self.practise_box.set_child_packing(self.g_lesson_heading, False, False, 0, 0)
         hbox.set_spacing(6)
-        hbox.pack_start(img)
-        self.g_deprecated_label = gtk.Label()
-        hbox.pack_start(self.g_deprecated_label)
-        self.practise_box.pack_start(hbox, False, False)
+        hbox.pack_start(img, True, True, 0)
+        self.g_deprecated_label = Gtk.Label()
+        hbox.pack_start(self.g_deprecated_label, True, True, 0)
+        self.practise_box.pack_start(hbox, False, False, 0)
         self.practise_box.reorder_child(hbox, 0)
     def set_deprecation_text(self, oldmodule, newmodule, filename):
         self.g_deprecated_label.set_line_wrap(True)
@@ -428,7 +432,8 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
              # does not have a music displayer. And the button will be
              # insensitive until the question has been solved.
              'show': _("_Show"),
-             'give_up': _("_Give up")
+             'give_up': _("_Give up"),
+             'backspace': _("Backspace"),
         }
         self._std_buttons.extend([x[0] for x in buttons])
         for b, cb in buttons:
@@ -444,7 +449,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         for b in ('repeat', 'repeat_slowly', 'repeat_arpeggio',
             'repeat_first', 'repeat_last',
             'repeat_melodic', 'guess_answer', 'play_tonic', 'play_music',
-            'play_answer',
+            'play_answer', 'backspace',
             'display_music', 'show', 'give_up'):
             if b in self._std_buttons:
                 getattr(self, 'g_%s' % b).set_sensitive(False)
@@ -524,6 +529,8 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.g_give_up.set_sensitive(False)
         if 'guess_answer' in self._std_buttons:
             self.g_guess_answer.set_sensitive(False)
+        if 'backspace' in self._std_buttons:
+            self.g_backspace.set_sensitive(False)
     def std_buttons_answer_wrong(self):
         if 'give_up' in self._std_buttons:
             self.g_give_up.set_sensitive(True)
@@ -538,6 +545,8 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.g_give_up.set_sensitive(False)
         if 'guess_answer' in self._std_buttons:
             self.g_guess_answer.set_sensitive(False)
+        if 'backspace' in self._std_buttons:
+            self.g_backspace.set_sensitive(False)
     def std_buttons_exception_cleanup(self):
         self._std_buttons_start_sensitivity()
     def on_cancel_test(self, *w):
@@ -575,6 +584,8 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         """
         self.handle_config_box_visibility()
         self.handle_statistics_page_sensibility()
+        if self.get_bool("lock-to-key"):
+            self.set_bool("lock-to-key", False)
         if hasattr(self, 'g_flashbar'):
             self.g_flashbar.require_size(["|Xy"])
         if self.m_t.m_P.header.lesson_heading:
@@ -609,18 +620,18 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             pass
     def on_key_press_event(self, widget, event):
         if (self.g_notebook is None or self.g_notebook.get_current_page() == 0) \
-           and event.type == gtk.gdk.KEY_PRESS:
+                and event.type == Gdk.EventType.KEY_PRESS:
             for s in self.m_key_bindings:
                 if self.keymatch(event, s):
                     self.m_key_bindings[s]()
                     return 1
     def keymatch(self, event, cfname):
         a, b = gu.parse_key_string(self.get_string(cfname))
-        return ((event.state & (gtk.gdk.CONTROL_MASK|gtk.gdk.SHIFT_MASK|gtk.gdk.MOD1_MASK)) == a) and (event.keyval == b)
+        return ((event.get_state() & (Gdk.ModifierType.CONTROL_MASK|Gdk.ModifierType.SHIFT_MASK|Gdk.ModifierType.MOD1_MASK)) == a) and (event.keyval == b)
     def setup_statisticsviewer(self, viewclass, heading):
         self.g_statview = viewclass(self.m_t.m_statistics, heading)
         self.g_statview.show()
-        self.g_notebook.append_page(self.g_statview, gtk.Label(_("Statistics")))
+        self.g_notebook.append_page(self.g_statview, Gtk.Label(label=_("Statistics")))
         self.g_notebook.connect('switch_page', self.on_switch_page)
     def on_switch_page(self, notebook, obj, pagenum):
         if pagenum == 2:
@@ -631,22 +642,22 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
     def _add_auto_new_question_gui(self, box):
         hbox = gu.bHBox(box, False)
         hbox.set_spacing(gu.PAD_SMALL)
-        adj = gtk.Adjustment(0, 0, 10, 0.1, 1)
+        adj = Gtk.Adjustment(0, 0, 10, 0.1, 1)
         spin = gu.nSpinButton(self.m_exname, 'seconds_before_new_question',
                        adj)
         spin.set_digits(1)
-        label = gtk.Label(_("Delay (seconds):"))
+        label = Gtk.Label(label=_("Delay (seconds):"))
         label.show()
         def f(button, spin=spin, label=label):
             spin.set_sensitive(button.get_active())
             label.set_sensitive(button.get_active())
         b = gu.nCheckButton(self.m_exname, 'new_question_automatically',
                             _("_New question automatically."), callback=f)
-        hbox.pack_start(b, False)
+        hbox.pack_start(b, False, False, 0)
         label.set_sensitive(b.get_active())
-        hbox.pack_start(label, False)
+        hbox.pack_start(label, False, False, 0)
         spin.set_sensitive(b.get_active())
-        hbox.pack_start(spin, False)
+        hbox.pack_start(spin, False, False, 0)
     def _lessonfile_exception(self, exception, sourcefile, lineno):
         m = gu.ExceptionDialog(exception)
         idx = self.m_t.m_P._idx
@@ -729,37 +740,36 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
 
 class RhythmAddOnGuiClass(object):
     def add_select_elements_gui(self):
-        self.g_element_frame = frame = gtk.Frame(_("Rhythms to use in question"))
-        self.config_box.pack_start(frame, False)
+        self.g_element_frame = frame = Gtk.Frame(label=_("Rhythms to use in question"))
+        self.config_box.pack_start(frame, False, False, 0)
         self.g_select_rhythms_box = gu.NewLineBox()
         self.g_select_rhythms_box.set_border_width(gu.PAD_SMALL)
         frame.add(self.g_select_rhythms_box)
     def add_select_num_beats_gui(self):
         ###
-        hbox = gtk.HBox()
+        hbox = Gtk.HBox()
         hbox.set_spacing(gu.hig.SPACE_SMALL)
-        label = gtk.Label(_("Number of beats in question:"))
-        hbox.pack_start(label, False)
+        label = Gtk.Label(label=_("Number of beats in question:"))
+        hbox.pack_start(label, False, False, 0)
         self.config_box_sizegroup.add_widget(label)
         label.set_alignment(1.0, 0.5)
         hbox.pack_start(gu.nSpinButton(self.m_exname, "num_beats",
-                     gtk.Adjustment(4, 1, 100, 1, 10)), False)
-        self.config_box.pack_start(hbox, False)
+                     Gtk.Adjustment(4, 1, 100, 1, 10)), False, False, 0)
+        self.config_box.pack_start(hbox, False, False, 0)
         hbox.show_all()
         #
-        hbox = gtk.HBox()
+        hbox = Gtk.HBox()
         hbox.set_spacing(gu.hig.SPACE_SMALL)
-        label = gtk.Label(_("Count in before question:"))
-        hbox.pack_start(label, False)
+        label = Gtk.Label(label=_("Count in before question:"))
+        hbox.pack_start(label, False, False, 0)
         self.config_box_sizegroup.add_widget(label)
         label.set_alignment(1.0, 0.5)
-        hbox.pack_start(label, False)
         hbox.pack_start(gu.nSpinButton(self.m_exname, "count_in",
-                     gtk.Adjustment(2, 0, 10, 1, 10)), False)
+                     Gtk.Adjustment(2, 0, 10, 1, 10)), False, False, 0)
         hbox.show_all()
-        self.config_box.pack_start(hbox, False)
+        self.config_box.pack_start(hbox, False, False, 0)
     def pngcheckbutton(self, i):
-        btn = gtk.CheckButton()
+        btn = Gtk.CheckButton()
         btn.add(gu.create_rhythm_image(const.RHYTHMS[i]))
         btn.show()
         btn.connect('clicked', self.select_element_cb, i)
@@ -814,7 +824,7 @@ class IntervalGui(Gui):
 
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
-        self.practise_box.pack_start(self.g_flashbar, False)
+        self.practise_box.pack_start(self.g_flashbar, False, False, 0)
         self.practise_box.set_spacing(gu.PAD)
 
         self.std_buttons_add(('new-interval', self.new_question),
@@ -829,7 +839,7 @@ class IntervalGui(Gui):
         hbox.set_spacing(gu.PAD_SMALL)
         gu.bLabel(hbox, _("Input interface:"), False)
 
-        combo = gtk.combo_box_new_text()
+        combo = Gtk.ComboBoxText()
         for i in range(len(inputwidgets.inputwidget_names)):
             combo.append_text(inputwidgets.inputwidget_names[i])
         if self.get_int('inputwidget') < len(inputwidgets.inputwidget_names):
@@ -837,32 +847,30 @@ class IntervalGui(Gui):
         else:
             combo.set_active(0)
         combo.connect('changed', lambda w: self.use_inputwidget(w.get_active()))
-        hbox.pack_start(combo, False)
+        hbox.pack_start(combo, False, False, 0)
 
         self.g_disable_unused_buttons = gu.nCheckButton(self.m_exname,
                     'disable_unused_intervals', _("_Disable unused buttons"))
-        hbox.pack_start(self.g_disable_unused_buttons)
+        hbox.pack_start(self.g_disable_unused_buttons, True, True, 0)
     def add_lock_to_key_gui(self):
         # gui to lock to a key
         def toggle_lock_to_key_sensitivity(checkbutton):
             self.g_notename.set_sensitive(checkbutton.get_active())
             self.g_scaletype.set_sensitive(checkbutton.get_active())
-        self.g_lock_to_key_hbox = gtk.HBox()
-        self.config_box.pack_start(self.g_lock_to_key_hbox, False)
+        self.g_lock_to_key_hbox = Gtk.HBox(False, gu.hig.SPACE_SMALL)
+        self.config_box.pack_start(self.g_lock_to_key_hbox, False, False, 0)
         check = gu.nCheckButton(self.m_exname, 'lock-to-key',
             _("Lock intervals to key:"),
             callback = toggle_lock_to_key_sensitivity)
-        self.g_lock_to_key_hbox.pack_start(check, False)
+        self.g_lock_to_key_hbox.pack_start(check, False, False, 0)
         self.g_notename = gu.nComboBox(self.m_exname, 'lock-to-key-note',
             mpd.MusicalPitch.new_from_int(60).get_user_notename(),
             [mpd.MusicalPitch.new_from_int(60 + x).get_user_notename() for x in range(12)])
         self.g_notename.show()
-        self.g_lock_to_key_hbox.pack_start(self.g_notename, False)
-        self.g_scaletype = gu.nComboBox(self.m_exname, 'lock-to-key-scaletype', _("Major"),
-            [_("Major"), _("Natural Minor"), _("Harmonic Minor")])
+        self.g_lock_to_key_hbox.pack_start(self.g_notename, False, False, 0)
+        self.g_scaletype = gu.nComboBox(self.m_exname, 'lock-to-key-scaletype', _("Major"), [n['name'] for n in utils.key_data.values()])
         self.g_scaletype.show()
-        self.g_lock_to_key_hbox.pack_start(self.g_scaletype, False)
-        self.g_lock_to_key_hbox.set_no_show_all(True)
+        self.g_lock_to_key_hbox.pack_start(self.g_scaletype, False, False, 0)
         toggle_lock_to_key_sensitivity(check)
     def select_inputwidget(self):
         """
@@ -885,22 +893,18 @@ class IntervalGui(Gui):
             v = []
             for x in range(self.get_int('maximum_number_of_intervals')):
                 v.append('ask_for_intervals_%i' % x)
-        if i == 0:
-            assert inputwidgets.inputwidget_names[i] == _("Buttons")
-            self.g_input = inputwidgets.IntervalButtonsWidget(self.m_exname,
-                  'intervals', self.click_on_interval, self.get_interval_input_list, v)
-        else:
-            self.g_input = inputwidgets.name_to_inputwidget(
-                                 inputwidgets.inputwidget_names[i],
-                                 self.click_on_interval)
-        self.practise_box.pack_start(self.g_input)
+        self.g_input = inputwidgets.int_to_inputwidget(
+            i, self.click_on_interval, self.m_exname,
+            self.get_interval_input_list, v)
+        self.practise_box.pack_start(self.g_input, True, True, 0)
         self.practise_box.reorder_child(self.g_input, 1)
         self.g_input.show()
         if self.m_t.m_tonika:
             # Don't call on_end_practise if we are starting up the exercise.
             # This whole thing is a mess.
             self.on_end_practise()
-        self.g_disable_unused_buttons.set_sensitive(self.get_int('inputwidget')==0)
+        self.g_disable_unused_buttons.set_sensitive(
+            isinstance(self.g_input, inputwidgets.IntervalButtonsWidgetBase))
     def setup_key_bindings(self):
         keys = ['minor2', 'major2', 'minor3', 'major3',
                 'perfect4', 'diminished5', 'perfect5', 'minor6',
@@ -919,22 +923,22 @@ class LessonbasedGui(Gui):
         Gui.__init__(self, teacher, no_notebook)
     def add_random_transpose_gui(self):
         self.g_random_transpose_box = hbox = gu.bHBox(self.config_box, False, False)
-        label = gtk.Label(_("Random transpose:"))
+        label = Gtk.Label(label=_("Random transpose:"))
         label.show()
-        hbox.pack_start(label, False)
+        hbox.pack_start(label, False, False, 0)
         hbox.set_spacing(6)
-        self.g_random_transpose = gtk.Label()
+        self.g_random_transpose = Gtk.Label()
         self.g_random_transpose.show()
-        hbox.pack_start(self.g_random_transpose)
+        hbox.pack_start(self.g_random_transpose, True, True, 0)
 
-        button = gtk.Button(_("Change ..."))
+        button = Gtk.Button(_("Change ..."))
         button.show()
         button.connect('clicked', self.run_random_transpose_dialog)
-        hbox.pack_start(button)
+        hbox.pack_start(button, True, True, 0)
     def run_random_transpose_dialog(self, widget):
         dlg = RandomTransposeDialog(self.m_t.m_P.header.random_transpose, solfege.win)
         response = dlg.run()
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             self.m_t.m_P.header.random_transpose = dlg.get_value()
             if self.m_t.m_P.header.random_transpose[0] == True:
                 self.g_random_transpose.set_text(_("Yes"))
