@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008 Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2011 Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
 from __future__ import absolute_import
 
-import gobject
+from gi.repository import GObject
 
 from solfege import abstract
 from solfege import gu
@@ -37,6 +37,12 @@ class Teacher(abstract.Teacher):
         abstract.Teacher.__init__(self, exname)
         self.lessonfileclass = lessonfile.IdByNameLessonfile
         self.m_statistics = statistics.LessonStatistics(self)
+        for s in ('vertic', 'horiz',
+                  'progression', # depcrecated, used to set labelformat
+                  'show', 'play', # at_music_start = show, play
+                  'accidentals', 'key', 'semitones', 'atonal',
+            ):
+            self.m_lessonfile_defs[s] = s
     def enter_test_mode(self):
         self.m_custom_mode = False
         self.m_statistics.enter_test_mode()
@@ -51,7 +57,7 @@ class Teacher(abstract.Teacher):
         UI will never call this function unless we have a usable lessonfile.
         """
         if self.m_timeout_handle:
-            gobject.source_remove(self.m_timeout_handle)
+            GObject.source_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
 
         if solfege.app.m_test_mode:
@@ -95,13 +101,13 @@ class Gui(abstract.LessonbasedGui):
         # practise_box #
         ################
         self.g_music_displayer = mpd.MusicDisplayer()
-        self.practise_box.pack_start(self.g_music_displayer, False)
+        self.practise_box.pack_start(self.g_music_displayer, False, False, 0)
         self.g_bb = QuestionNameButtonTable(self.m_t.m_exname)
-        self.practise_box.pack_start(self.g_bb, False)
+        self.practise_box.pack_start(self.g_bb, False, False, 0)
 
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
-        self.practise_box.pack_start(self.g_flashbar, False)
+        self.practise_box.pack_start(self.g_flashbar, False, False, 0)
         self.practise_box.set_spacing(gu.PAD)
 
         self.std_buttons_add(
@@ -126,10 +132,10 @@ class Gui(abstract.LessonbasedGui):
         # -----------------------------------------
         self.g_select_questions_category_box, category_box= gu.hig_category_vbox(
             _("Questions to ask"))
-        self.config_box.pack_start(self.g_select_questions_category_box, True)
+        self.config_box.pack_start(self.g_select_questions_category_box, False, False, 0)
         self.g_select_questions = QuestionNameCheckButtonTable(self.m_t)
         self.g_select_questions.initialize(4, 0)
-        category_box.pack_start(self.g_select_questions, False)
+        category_box.pack_start(self.g_select_questions, False, False, 0)
         self.g_select_questions.show()
         # ------------------------------------------
         self._add_auto_new_question_gui(self.config_box)
@@ -144,14 +150,11 @@ class Gui(abstract.LessonbasedGui):
             self.setup_action_area_buttons()
         self.add_watch('ask_for_names', _f)
     def update_answer_buttons(self):
-        if self.m_t.m_P:
-            self.g_bb.initialize(self.m_t.m_P.header.fillnum,
-                                 self.m_t.m_P.header.filldir)
-            for question in self.m_t.m_P.iterate_questions_with_unique_names():
-                self.g_bb.add(question, self.on_click)
-        else:
-            self.g_bb.initialize(0, 0)
-        if self.m_t.m_P and self.m_t.m_custom_mode:
+        self.g_bb.initialize(self.m_t.m_P.header.fillnum,
+                             self.m_t.m_P.header.filldir)
+        for question in self.m_t.m_P.iterate_questions_with_unique_names():
+            self.g_bb.add(question, self.on_click)
+        if self.m_t.m_custom_mode:
             self.g_bb.ask_for_names_changed()
     def update_select_question_buttons(self):
         #FIXME duplicate code in src/chord.py
@@ -214,11 +217,11 @@ class Gui(abstract.LessonbasedGui):
             if 'set' in self.m_t.m_P.get_question():
                 for idx, question in enumerate(self.m_t.m_P.m_questions):
                     if question.set == self.m_t.m_P.get_question().set \
-                        and question.name.cval == button.get_data('cname'):
+                        and question.name.cval == button.m_cname:
                         self.m_t.m_P.play_question(question)
                         return
             for idx, question in enumerate(self.m_t.m_P.m_questions):
-                if question.name.cval == button.get_data('cname'):
+                if question.name.cval == button.m_cname:
                     self.m_t.m_P.play_question(question)
                     return
         except Exception, e:
@@ -233,12 +236,12 @@ class Gui(abstract.LessonbasedGui):
             return
         try:
             if self.m_t.q_status == self.QSTATUS_SOLVED:
-                if self.m_t.guess_answer(button.get_data('cname')):
+                if self.m_t.guess_answer(button.m_cname):
                     self.g_flashbar.flash(_("Correct, but you have already solved this question"))
                 else:
                     self.g_flashbar.flash(_("Wrong, but you have already solved this question"))
             elif self.m_t.q_status in (self.QSTATUS_NEW, self.QSTATUS_WRONG):
-                if self.m_t.guess_answer(button.get_data('cname')):
+                if self.m_t.guess_answer(button.m_cname):
                     self.g_flashbar.flash(_("Correct"))
                     self.std_buttons_answer_correct()
                     if self.m_t.m_P.header.have_music_displayer:
@@ -287,7 +290,6 @@ class Gui(abstract.LessonbasedGui):
         self.setup_action_area_buttons()
         self.update_answer_buttons()
         self.update_select_question_buttons()
-        self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
         self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
         self.g_flashbar.require_size([
             _("Right click is not allowed for this lesson file."),
