@@ -1,3 +1,4 @@
+# vim: set fileencoding=utf-8:
 # GNU Solfege - free ear training software
 # Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
@@ -16,7 +17,7 @@
 
 from __future__ import absolute_import
 
-import gtk
+from gi.repository import Gtk
 
 from solfege import abstract
 from solfege import gu
@@ -96,6 +97,7 @@ class Teacher(abstract.MelodicIntervalTeacher):
         self.m_tonika = None
 
 class Gui(abstract.IntervalGui):
+    lesson_heading = _("Identify the interval")
     def __init__(self, teacher):
         abstract.IntervalGui.__init__(self, teacher)
         self.g_test_stat_dlg = None
@@ -104,12 +106,14 @@ class Gui(abstract.IntervalGui):
         # config_box #
         ##############
         self.g_mici = MultipleIntervalConfigWidget(self.m_exname)
-        self.config_box.pack_start(self.g_mici, False, False)
-        self.config_box.pack_start(gtk.HBox(), False,
+        self.config_box.pack_start(self.g_mici, False, False, 0)
+        box = Gtk.HBox(False, 0)
+        self.config_box.pack_start(box, False, False,
                                    padding=gu.PAD_SMALL)
         self._add_auto_new_question_gui(self.config_box)
         # ----------------------------------
         self._create_select_inputwidget_gui()
+        self.add_lock_to_key_gui()
         self.config_box.show_all()
         ##############
         # statistics #
@@ -186,8 +190,8 @@ class Gui(abstract.IntervalGui):
         if self.m_t.q_status == self.QSTATUS_SOLVED:
             self.g_flashbar.flash(_("You have already identified this interval"))
             return
-        if not (-17 < interval < 17):
-            self.g_flashbar.flash(_("Ignoring intervals greater than major tenth."))
+        if not (mpd.interval.min_interval < interval <= mpd.interval.max_interval):
+            self.g_flashbar.flash(_("Ignoring intervals greater than double octave."))
             self.g_input.forget_last_tone()
             return
         self.m_answer.append(interval)
@@ -224,17 +228,21 @@ class Gui(abstract.IntervalGui):
         self.m_number_of_intervals_in_question \
                = self.get_int('number_of_intervals')
         self.m_answer = []
-        if isinstance(self.g_input, inputwidgets.IntervalButtonsWidget):
-            ret = self.m_t.new_question(self.get_string('user/lowest_pitch'),
-                                    self.get_string('user/highest_pitch'))
+        if hasattr(self.g_input, 'use_users_vocal_range'):
+            low = self.get_string('user/lowest_pitch')
+            high = self.get_string('user/highest_pitch')
         else:
-            ret = self.m_t.new_question(self.g_input.m_lowest_tone,
-                                    self.g_input.m_highest_tone)
-        if ret == Teacher.ERR_CONFIGURE:
+            low = self.g_input.m_lowest_tone
+            high = self.g_input.m_highest_tone
+        try:
+            ret = self.m_t.new_question(low, high)
+        except Teacher.ConfigureException, e:
             self.g_flashbar.clear()
-            self.g_flashbar.flash(_("The exercise has to be better configured."))
+            solfege.win.display_error_message2(_("Exercise configuration problem"), unicode(e))
             self.g_repeat.set_sensitive(False)
-        elif ret == Teacher.OK:
+        else:
+            if ret == Teacher.ERR_PICKY:
+               self.g_flashbar.flash(_("You have to solve this question first."))
             self.g_input.set_first_note(self.m_t.m_tonika)
             def exception_cleanup():
                 self.m_t.end_practise()
@@ -250,15 +258,9 @@ class Gui(abstract.IntervalGui):
             #inputwidget 0 is always the buttons.
             if self.get_int('inputwidget') == 0:
                 self.g_input.grab_focus_first_sensitive_button()
-        elif ret == Teacher.ERR_PICKY:
-            self.g_flashbar.flash(_("You have to solve this question first."))
     def on_start_practise(self):
         self.m_t.start_practise()
         super(Gui, self).on_start_practise()
-        if self.m_t.m_P.header.lesson_heading:
-            self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
-        else:
-            self.set_lesson_heading(_("Identify the interval"))
         if self.m_t.m_custom_mode:
             self.g_mici.show()
         else:
@@ -272,6 +274,7 @@ class Gui(abstract.IntervalGui):
         else:
             self.g_flashbar.delayed_flash(self.short_delay,
                 _("Click 'New interval' to begin."))
+        self.g_lock_to_key_hbox.set_sensitive(self.get_int('number_of_intervals') == 1)
     def on_end_practise(self):
         self.m_t.end_practise()
         self.std_buttons_end_practise()
