@@ -17,10 +17,13 @@
 from __future__ import absolute_import
 from __future__ import division
 
-import gtk
+from gi.repository import Gtk
+from gi.repository import Gdk
+
 import math
 
 from solfege import cfg
+from solfege import gu
 from solfege import mpd
 from solfege import utils
 
@@ -32,18 +35,6 @@ class CairoCommon(object):
     def __init__(self):
         self.m_marks = []
         self.m_clicked_tones = []
-    def _redraw(self):
-        self.window.invalidate_rect(gtk.gdk.Rectangle(0, 0,
-            self.allocation.width, self.allocation.height), True)
-        return False
-    def expose(self, widget, event):
-        self.context = widget.window.cairo_create()
-        # set a clip region for the expose event
-        self.context.rectangle(event.area.x, event.area.y,
-                               event.area.width, event.area.height)
-        self.context.clip()
-        self.draw(self.context)
-        return False
     def clear(self):
         self.m_marks = []
         self.m_clicked_tones = []
@@ -54,7 +45,7 @@ class CairoCommon(object):
         """
         if midi_int not in self.m_marks:
             self.m_marks.append((midi_int, color))
-        self._redraw()
+        self.queue_draw()
     def set_first_note(self, note):
         self.clear()
         self.m_clicked_tones = [int(note)]
@@ -79,13 +70,13 @@ class CairoCommon(object):
                 self.m_clicked_tones.append(midi_int)
             self.m_callback(mouse_button, interval, midi_int)
 
-class PianoKeyboard(gtk.DrawingArea, CairoCommon):
+class PianoKeyboard(Gtk.DrawingArea, CairoCommon):
     def __init__(self, num_octaves, lowest_c, key_w=15):
-        gtk.DrawingArea.__init__(self)
+        Gtk.DrawingArea.__init__(self)
         CairoCommon.__init__(self)
-        self.connect("expose_event", self.expose)
+        self.connect("draw", self.draw)
         self.connect("button-press-event", self._on_button_press)
-        self.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+        self.set_events(Gdk.EventMask.BUTTON_PRESS_MASK)
         # Piano stuff
         self.m_num_octaves = num_octaves
         self.m_lowest_c = mpd.MusicalPitch.new_from_notename(lowest_c)
@@ -126,10 +117,10 @@ class PianoKeyboard(gtk.DrawingArea, CairoCommon):
         clicked_on.m_accidental_i = black_clicked
         clicked_on.m_octave_i += clicked_octave
         self.on_button_press_event(event, clicked_on.semitone_pitch())
-    def draw(self, context):
+    def draw(self, widget, context):
         num_key = 7 * self.m_num_octaves
         width = num_key * self.m_key_w
-        self.m_pos_x = pos_x = int((self.allocation.width - width) / 2) + 0.5
+        self.m_pos_x = pos_x = int((self.get_allocated_width() - width) / 2) + 0.5
         pos_y = .5
         context.set_line_width(1.2)
         context.save()
@@ -174,8 +165,8 @@ class PianoOctaveWithAccelName(PianoKeyboard):
         self.m_callback = callback
         self.m_visible_accels = False
         self.m_keys = keys
-    def draw(self, context):
-        PianoKeyboard.draw(self, context)
+    def draw(self, widget, context):
+        PianoKeyboard.draw(self, widget, context)
         if not self.m_visible_accels:
             return
         context.save()
@@ -221,30 +212,22 @@ class IntervalPianoWidget(PianoKeyboard):
         self._handle_tone_clicked(midi_int, event.button)
 
 
-class IntervalButtonsWidget(gtk.Table, cfg.ConfigUtils):
-    def __init__(self, exname, name, callback, sensicallback, vars_to_watch):
-        gtk.Table.__init__(self, 1, 1, True)
+class IntervalButtonsWidgetBase(Gtk.Grid, cfg.ConfigUtils):
+    use_users_vocal_range = True
+
+    def __init__(self, exname, callback, sensicallback):
+        """
+        callback -- the function to call when the user click on a button.
+        sensicallback -- a function that will return a list of all interval
+                         it will be asked for.
+        """
+        Gtk.Grid.__init__(self)
         cfg.ConfigUtils.__init__(self, exname)
-        self.m_name = name
+        self.set_orientation(Gtk.Orientation.VERTICAL)
         self.get_sensitive_buttons = sensicallback
         self.m_callback = callback
         self.m_buttons = {}
-        self.new_int_button(_("Minor Second"), 1, 0, 1, 0, 2)
-        self.new_int_button(_("Major Second"),  2, 0, 1, 2, 4)
-        self.new_int_button(_("Minor Third"),   3, 0, 1, 4, 6)
-        self.new_int_button(_("Major Third"),    4, 0, 1, 6, 8)
-        self.new_int_button(_("Perfect Fourth"),    5, 1, 2, 0, 2)
-        self.new_int_button(_("Tritone"),    6, 1, 2, 2, 4)
-        self.new_int_button(_("Perfect Fifth"),    7, 1, 2, 4, 6)
-        self.new_int_button(_("Minor Sixth"),  8, 1, 2, 6, 8)
-        self.new_int_button(_("Major Sixth"),   9, 2, 3, 0, 2)
-        self.new_int_button(_("Minor Seventh"),10, 2, 3, 2, 4)
-        self.new_int_button(_("Major Seventh"), 11, 2, 3, 4, 6)
-        self.new_int_button(_("Perfect Octave"),   12, 2, 3, 6, 8)
-        self.new_int_button(_("Minor Ninth"),  13, 3, 4, 0, 2)
-        self.new_int_button(_("Major Ninth"),   14, 3, 4, 2, 4)
-        self.new_int_button(_("Minor Tenth"), 15, 3, 4, 4, 6)
-        self.new_int_button(_("Major Tenth"),  16, 3, 4, 6, 8)
+    def post_construct(self, vars_to_watch):
         self.m_lowest_tone = mpd.LOWEST_NOTENAME
         self.m_highest_tone = mpd.HIGHEST_NOTENAME
         self.add_watch('disable_unused_intervals', self.intervals_changed)
@@ -255,27 +238,32 @@ class IntervalButtonsWidget(gtk.Table, cfg.ConfigUtils):
         if self.get_bool('disable_unused_intervals'):
             self.set_sensitivity(self.get_sensitive_buttons())
         else:
-            for x in range(1, 17):
-                self.m_buttons[x].set_sensitive(True)
+            self.set_sensitivity(range(mpd.interval.max_interval+1))
     def set_sensitivity(self, make_active):
-        for x in range(1, 17):
-            self.m_buttons[x].set_sensitive(x in make_active)
-    def new_int_button(self, txt, nr, x1, x2, y1, y2):
+        for x in range(1, mpd.interval.max_interval + 1):
+            if x in self.m_buttons:
+                self.m_buttons[x].set_sensitive(x in make_active)
+    def mk_button(self, nr):
         # buttonwidget calls m_callback with None as midi_int because it
         # does not know if you mean interval up or down when you click
         # the buttons
-        self.m_buttons[nr] = b = gtk.Button(txt)
-        w, h = b.size_request()
-        b.set_size_request(w, int(h*1.4))
-        l=b.get_child().set_justify(gtk.JUSTIFY_CENTER)
-        self.attach(b, x1, x2, y1, y2)
+        txt = mpd.Interval.new_from_int(nr).get_name()
+        self.m_buttons[nr] = b = Gtk.Button(txt)
         b.connect('clicked',
                   lambda s, nr=nr, self=self:self.m_callback(1, nr, None))
-        b.set_data('interval', nr)
-        b.connect('event', self._abc)
-    def _abc(self, button, event):
-        if event.type == gtk.gdk.BUTTON_RELEASE and event.button == 3:
-            self.m_callback(3, button.get_data('interval'), None)
+        b.connect('button-press-event', self._abc, nr)
+        b.set_vexpand(True)
+        b.set_hexpand(True)
+        return b
+    def row(self, *data):
+        row = Gtk.Grid()
+        row.set_column_homogeneous(True)
+        self.add(row)
+        for i in data:
+            row.add(self.mk_button(i))
+    def _abc(self, button, event, interval):
+        if event.button == 3:
+            self.m_callback(3, interval, None)
     def set_first_note(self, note):
         self.m_first_note = int(note)
     def know_directions(self):
@@ -295,10 +283,45 @@ class IntervalButtonsWidget(gtk.Table, cfg.ConfigUtils):
         """
         pass
 
-class AbstractGuitarWidget(gtk.DrawingArea, CairoCommon):
+class IntervalButtonsWidget(IntervalButtonsWidgetBase):
+    def __init__(self, exname, callback, sensicallback, vars_to_watch):
+        IntervalButtonsWidgetBase.__init__(self, exname, callback, sensicallback)
+        g1 = Gtk.Grid()
+        for i, (x, y) in enumerate((
+                (0, 0), (1, 0), (2, 0), (3, 0),
+                (0, 1), (1, 1), (2, 1), (3, 1),
+                (0, 2), (1, 2), (2, 2), (3, 2))):
+            g1.attach(self.mk_button(i + 1), x, y, 1, 1)
+        g2 = Gtk.Grid()
+        for i, (x, y) in enumerate((
+                (0, 0), (1, 0), (2, 0), (3, 0),
+                (0, 1), (1, 1), (2, 1), (3, 1),
+                (0, 2), (1, 2), (2, 2), (3, 2))):
+            g2.attach(self.mk_button(i + 13), x, y, 1, 1)
+        self.set_row_spacing(gu.hig.SPACE_MEDIUM)
+        self.add(g1)
+        self.add(g2)
+        self.post_construct(vars_to_watch)
+
+
+class IntervalButtonsWidget2(IntervalButtonsWidgetBase):
+    def __init__(self, exname, callback, sensicallback, vars_to_watch):
+        IntervalButtonsWidgetBase.__init__(self, exname, callback, sensicallback)
+        self.row(1, 2, 3, 4)
+        self.row(5, 6, 7)
+        self.row(8, 9, 10, 11)
+        self.row(12)
+        self.row(13, 14, 15, 16)
+        self.row(17, 18, 19)
+        self.row(20, 21, 22, 23)
+        self.row(24)
+        self.post_construct(vars_to_watch)
+
+
+class AbstractGuitarWidget(Gtk.DrawingArea, CairoCommon):
     def __init__(self, callback, strings,
                 string_thickness=(1, 1, 1, 1, 1, 1)):
-        gtk.DrawingArea.__init__(self)
+        Gtk.DrawingArea.__init__(self)
         CairoCommon.__init__(self)
         self.m_callback = callback
         self.m_strings = strings
@@ -329,8 +352,8 @@ class AbstractGuitarWidget(gtk.DrawingArea, CairoCommon):
         self.m_neckw = self.m_neckborder \
                        + (self.m_numstring-1)*self.m_stringdist \
                        + 1 + self.m_neckborder
-        self.set_events(gtk.gdk.BUTTON_PRESS_MASK|gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.POINTER_MOTION_MASK | gtk.gdk.ENTER_NOTIFY_MASK | gtk.gdk.LEAVE_NOTIFY_MASK)
-        self.connect('expose_event', self.expose)
+        self.set_events(Gdk.EventMask.BUTTON_PRESS_MASK|Gdk.EventMask.BUTTON_RELEASE_MASK | Gdk.EventMask.POINTER_MOTION_MASK | Gdk.EventMask.ENTER_NOTIFY_MASK | Gdk.EventMask.LEAVE_NOTIFY_MASK)
+        self.connect('draw', self.draw)
         self.m_mouse_pos = None, None
         self.connect('motion-notify-event', self.on_motion_notify_event)
         self.connect('leave-notify-event', self.on_leave_notify_event)
@@ -343,7 +366,7 @@ class AbstractGuitarWidget(gtk.DrawingArea, CairoCommon):
             self._handle_tone_clicked(midi_int, event.button)
     def on_leave_notify_event(self, widget, event):
         self.m_mouse_pos = None, None
-        self._redraw()
+        self.queue_draw()
     def on_motion_notify_event(self, widget, event):
         x, y = self.event2xy(event)
         need_redraw = False
@@ -356,10 +379,10 @@ class AbstractGuitarWidget(gtk.DrawingArea, CairoCommon):
                 need_redraw = True
             self.m_mouse_pos = None, None
         if need_redraw:
-            self._redraw()
-    def draw(self, context):
-        self.m_posx = int((self.allocation.width - self.m_neckl) / 2) + 0.5
-        self.m_posy = int((self.allocation.height - self.m_neckw) / 2) + 0.5
+            self.queue_draw()
+    def draw(self, widget, context):
+        self.m_posx = int((self.get_allocated_width() - self.m_neckl) / 2) + 0.5
+        self.m_posy = int((self.get_allocated_height() - self.m_neckw) / 2) + 0.5
         context.set_line_width(1.2)
         context.save()
         LG = 0.75 # white grey
@@ -498,9 +521,9 @@ class GuitarWidget(AbstractGuitarWidget):
         AbstractGuitarWidget.__init__(self, callback, strings, string_thickness)
 
 
-class AccordionWidget(gtk.DrawingArea, CairoCommon):
+class AccordionWidget(Gtk.DrawingArea, CairoCommon):
     def __init__(self, callback, keyboard_system):
-        gtk.DrawingArea.__init__(self)
+        Gtk.DrawingArea.__init__(self)
         CairoCommon.__init__(self)
         self.m_callback = callback
         def ff(first, count):
@@ -531,9 +554,9 @@ class AccordionWidget(gtk.DrawingArea, CairoCommon):
         self.m_button_xdist = 20
         self.m_button_ydist = 18
 
-        self.connect('expose_event', self.expose)
+        self.connect('draw', self.draw)
         self.connect('button-press-event', self._on_button_press)
-        self.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+        self.set_events(Gdk.EventMask.BUTTON_PRESS_MASK)
     def event_to_button_pos(self, event):
         """
         Return a tuple telling which button on the accordion that
@@ -556,7 +579,7 @@ class AccordionWidget(gtk.DrawingArea, CairoCommon):
             return
         midi_int = mpd.notename_to_int(self.m_notenames[y][x])
         self._handle_tone_clicked(midi_int, event.button)
-    def draw(self, context):
+    def draw(self, widget, context):
         # number of buttons, horiz and vertic.
         self.m_bx = bx = len(self.m_notenames[0])
         self.m_by = by = len(self.m_notenames)
@@ -565,8 +588,8 @@ class AccordionWidget(gtk.DrawingArea, CairoCommon):
         #
         w = bx * self.m_button_xdist + bb * 2
         h = by * self.m_button_ydist + bb * 2
-        self.m_posx = posx = int((self.allocation.width - w) / 2) + 0.5
-        self.m_posy = posy = int((self.allocation.height - h) / 2) + 0.5
+        self.m_posx = posx = int((self.get_allocated_width() - w) / 2) + 0.5
+        self.m_posy = posy = int((self.get_allocated_height() - h) / 2) + 0.5
         context.set_line_width(1.2)
         context.save()
         context.move_to(posx, posy)
@@ -620,35 +643,40 @@ class AccordionWidget(gtk.DrawingArea, CairoCommon):
 
 
 inputwidget_names = [_("Buttons"),
+                 _("Buttons %i") % 2,
                  _("Piano"), _("Guitar"), _("Bass"),
                  _("5 string bass"), _("6 string bass"),
                  _("Accordion B griff"), _("Accordion C griff"),
                  _("Accordion (system used in Finland)"),
                  _("Accordion (Belgian Charleroi do2)")]
 
-def name_to_inputwidget(name, callback):
-    if name == _("Piano"):
+def int_to_inputwidget(i, callback, exname, get_interval_input_list, vars_to_watch):
+    if i in (0, 1):
+        w = {0: IntervalButtonsWidget,
+             1: IntervalButtonsWidget2}[i](exname, callback,
+                                           get_interval_input_list,
+                                           vars_to_watch)
+    elif i == 2:
         w = IntervalPianoWidget(callback)
-    elif name == _("Accordion B griff"):
-        w = AccordionWidget(callback, 'norwegian')
-    elif name == _("Accordion C griff"):
-        w = AccordionWidget(callback, 'swedish')
-    elif name == _("Accordion (system used in Finland)"):
-        w = AccordionWidget(callback, 'finnish')
-    elif name == _("Accordion (Belgian Charleroi do2)"):
-        w = AccordionWidget(callback, 'belgian')
-    elif name == _("Guitar"):
+    elif i == 3:
         w = GuitarWidget(callback, ["e'", "b", "g", "d", "a,", "e,"],
                                    (1, 1, 1, 2, 2, 3))
-    elif name == _("Bass"):
+    elif i == 4:
         w = AbstractGuitarWidget(callback, ["g,", "d,", "a,,", "e,,"],
                                  (1, 1, 2, 3))
-    elif name == _("5 string bass"):
+    elif i == 5:
         w = AbstractGuitarWidget(callback, ["g,", "d,", "a,,", "e,,", "b,,,"],
                                  (1, 1, 2, 3, 3))
-    else:
-        assert name == _("6 string bass")
+    elif i == 6:
         w = AbstractGuitarWidget(callback, ["c", "g,", "d,", "a,,", "e,,", "b,,,"],
                                  (1, 1, 2, 2, 3, 3))
+    elif i == 7:
+        w = AccordionWidget(callback, 'norwegian')
+    elif i == 8:
+        w = AccordionWidget(callback, 'swedish')
+    elif i == 9:
+        w = AccordionWidget(callback, 'finnish')
+    elif i == 10:
+        w = AccordionWidget(callback, 'belgian')
     return w
 
