@@ -19,7 +19,7 @@ from __future__ import absolute_import
 This file does the checking for optional features that depend on
 python modules the user can have installed.
 
-It also does sanity check on the python and pygtk versions and some
+It also does sanity check on the python and gtk versions and some
 other initial setup tasks.
 """
 
@@ -31,22 +31,11 @@ def assert_python_version(required_version):
     if sys.version_info < required_version:
         sys.exit("Solfege need Python %s or newer. The configure script told you so.\nThis is Python %s" % (".".join([str(i) for i in required_version]), sys.version))
 
-def setup_pygtk(required_version):
-    import pygtk
+def init(options):
     # this is needed for py2exe
     if sys.platform == 'win32':
         os.environ['PATH'] += ";lib;bin;"
-    else:
-        pygtk.require("2.0")
-    import gtk
-    gtk.rc_parse("solfege.gtkrc")
-    # The rest of this function is just for sanity check. Not really required.
-    if gtk.pygtk_version < required_version:
-        sys.exit("\n" + "\n ".join(textwrap.wrap(" GNU Solfege requires pygtk version %s or newer. The version installed appears to be %s. Exiting program." % (
-            ".".join([str(i) for i in required_version]),
-            ".".join([str(i) for i in gtk.pygtk_version])))))
-
-def init(options):
-    assert_python_version((2, 5))
-    setup_pygtk((2, 12))
+    from gi.repository import Gtk
+    assert_python_version((2, 7))
+    Gtk.check_version(3, 4, 0)
 
